@@ -4,7 +4,8 @@ from __future__ import annotations
 
 import json
 import os
-import warnings
+import re
+from collections import defaultdict
 from typing import Any, Callable, Generator, Optional
 
 import cv2
@@ -36,11 +37,14 @@ class Texture(Component):
         """Class which represents a layer with textures and tags.
         It's using to obtain data from OSM using tags and make changes into corresponding textures.
 
-        Args:
+        Arguments:
             name (str): Name of the layer.
             tags (dict[str, str | list[str]]): Dictionary of tags to search for.
             width (int | None): Width of the polygon in meters (only for LineString).
             color (tuple[int, int, int]): Color of the layer in BGR format.
+            exclude_weight (bool): Flag to exclude weight from the texture.
+            priority (int | None): Priority of the layer.
+            info_layer (str | None): Name of the corresnponding info layer.
 
         Attributes:
             name (str): Name of the layer.
@@ -58,6 +62,9 @@ class Texture(Component):
             color: tuple[int, int, int] | list[int] | None = None,
             exclude_weight: bool = False,
             priority: int | None = None,
+            info_layer: str | None = None,
+            usage: str | None = None,
+            background: bool = False,
         ):
             self.name = name
             self.count = count
@@ -66,6 +73,9 @@ class Texture(Component):
             self.color = color if color else (255, 255, 255)
             self.exclude_weight = exclude_weight
             self.priority = priority
+            self.info_layer = info_layer
+            self.usage = usage
+            self.background = background
 
         def to_json(self) -> dict[str, str | list[str] | bool]:  # type: ignore
             """Returns dictionary with layer data.
@@ -80,6 +90,9 @@ class Texture(Component):
                 "color": list(self.color),
                 "exclude_weight": self.exclude_weight,
                 "priority": self.priority,
+                "info_layer": self.info_layer,
+                "usage": self.usage,
+                "background": self.background,
             }
 
             data = {k: v for k, v in data.items() if v is not None}
@@ -89,7 +102,7 @@ class Texture(Component):
         def from_json(cls, data: dict[str, str | list[str] | bool]) -> Texture.Layer:
             """Creates a new instance of the class from dictionary.
 
-            Args:
+            Arguments:
                 data (dict[str, str | list[str] | bool]): Dictionary with layer data.
 
             Returns:
@@ -110,29 +123,29 @@ class Texture(Component):
             weight_postfix = "_weight" if not self.exclude_weight else ""
             return os.path.join(weights_directory, f"{self.name}{idx}{weight_postfix}.png")
 
-        def path_preview(self, previews_directory: str) -> str:
+        def path_preview(self, weights_directory: str) -> str:
             """Returns path to the preview of the first texture of the layer.
 
             Arguments:
-                previews_directory (str): Path to the directory with previews.
+                weights_directory (str): Path to the directory with weights.
 
             Returns:
                 str: Path to the preview.
             """
-            return self.path(previews_directory).replace(".png", "_preview.png")
+            return self.path(weights_directory).replace(".png", "_preview.png")
 
-        def get_preview_or_path(self, previews_directory: str) -> str:
+        def get_preview_or_path(self, weights_directory: str) -> str:
             """Returns path to the preview of the first texture of the layer if it exists,
             otherwise returns path to the texture.
 
             Arguments:
-                previews_directory (str): Path to the directory with previews.
+                weights_directory (str): Path to the directory with weights.
 
             Returns:
                 str: Path to the preview or texture.
             """
-            preview_path = self.path_preview(previews_directory)
-            return preview_path if os.path.isfile(preview_path) else self.path(previews_directory)
+            preview_path = self.path_preview(weights_directory)
+            return preview_path if os.path.isfile(preview_path) else self.path(weights_directory)
 
         def paths(self, weights_directory: str) -> list[str]:
             """Returns a list of paths to the textures of the layer.
@@ -145,31 +158,51 @@ class Texture(Component):
                 list[str]: List of paths to the textures.
             """
             weight_files = os.listdir(weights_directory)
+
+            # Inconsistent names are the name of textures that are not following the pattern
+            # of texture_name{idx}_weight.png.
+            inconsistent_names = ["forestRockRoot", "waterPuddle"]
+
+            if self.name in inconsistent_names:
+                return [
+                    os.path.join(weights_directory, weight_file)
+                    for weight_file in weight_files
+                    if weight_file.startswith(self.name)
+                ]
+
             return [
                 os.path.join(weights_directory, weight_file)
                 for weight_file in weight_files
-                if weight_file.startswith(self.name)
+                if re.match(rf"{self.name}\d{{2}}_weight.png", weight_file)
             ]
 
     def preprocess(self) -> None:
-        self.light_version = self.kwargs.get("light_version", False)
-        self.logger.debug("Light version: %s.", self.light_version)
+        """Preprocesses the data before the generation."""
+        custom_schema = self.kwargs.get("texture_custom_schema")
+        if custom_schema:
+            layers_schema = custom_schema  # type: ignore
+            self.logger.info("Custom schema loaded with %s layers.", len(layers_schema))
+        else:
+            if not os.path.isfile(self.game.texture_schema):
+                raise FileNotFoundError(
+                    f"Texture layers schema not found: {self.game.texture_schema}"
+                )
 
-        if not os.path.isfile(self.game.texture_schema):
-            raise FileNotFoundError(f"Texture layers schema not found: {self.game.texture_schema}")
+            try:
+                with open(self.game.texture_schema, "r", encoding="utf-8") as f:
+                    layers_schema = json.load(f)
+            except json.JSONDecodeError as e:
+                raise ValueError(f"Error loading texture layers schema: {e}") from e
 
         try:
-            with open(self.game.texture_schema, "r", encoding="utf-8") as f:
-                layers_schema = json.load(f)
-        except json.JSONDecodeError as e:
-            raise ValueError(f"Error loading texture layers schema: {e}") from e
-
-        self.layers = [self.Layer.from_json(layer) for layer in layers_schema]
-        self.logger.info("Loaded %s layers.", len(self.layers))
+            self.layers = [self.Layer.from_json(layer) for layer in layers_schema]  # type: ignore
+            self.logger.info("Loaded %s layers.", len(self.layers))
+        except Exception as e:  # pylint: disable=W0703
+            raise ValueError(f"Error loading texture layers: {e}") from e
 
         base_layer = self.get_base_layer()
         if base_layer:
-            self.logger.info("Base layer found: %s.", base_layer.name)
+            self.logger.debug("Base layer found: %s.", base_layer.name)
         else:
             self.logger.warning("No base layer found.")
 
@@ -178,6 +211,9 @@ class Texture(Component):
         self.info_save_path = os.path.join(self.map_directory, "generation_info.json")
         self.logger.debug("Generation info save path: %s.", self.info_save_path)
 
+        self.info_layer_path = os.path.join(self.info_layers_directory, "textures.json")
+        self.logger.debug("Info layer path: %s.", self.info_layer_path)
+
     def get_base_layer(self) -> Layer | None:
         """Returns base layer.
 
@@ -189,10 +225,57 @@ class Texture(Component):
                 return layer
         return None
 
+    def get_background_layers(self) -> list[Layer]:
+        """Returns list of background layers.
+
+        Returns:
+            list[Layer]: List of background layers.
+        """
+        return [layer for layer in self.layers if layer.background]
+
+    def get_layer_by_usage(self, usage: str) -> Layer | None:
+        """Returns layer by usage.
+
+        Arguments:
+            usage (str): Usage of the layer.
+
+        Returns:
+            Layer | None: Layer.
+        """
+        for layer in self.layers:
+            if layer.usage == usage:
+                return layer
+        return None
+
     def process(self):
         self._prepare_weights()
         self._read_parameters()
         self.draw()
+        self.rotate_textures()
+
+    def rotate_textures(self) -> None:
+        """Rotates textures of the layers which have tags."""
+        if self.rotation:
+            # Iterate over the layers which have tags and rotate them.
+            for layer in self.layers:
+                if layer.tags:
+                    self.logger.debug("Rotating layer %s.", layer.name)
+                    layer_paths = layer.paths(self._weights_dir)
+                    layer_paths += [layer.path_preview(self._weights_dir)]
+                    for layer_path in layer_paths:
+                        if os.path.isfile(layer_path):
+                            self.rotate_image(
+                                layer_path,
+                                self.rotation,
+                                output_height=self.map_size,
+                                output_width=self.map_size,
+                            )
+                        else:
+                            self.logger.warning("Layer path %s not found.", layer_path)
+                else:
+                    self.logger.debug(
+                        "Skipping rotation of layer %s because it has no tags.", layer.name
+                    )
 
     # pylint: disable=W0201
     def _read_parameters(self) -> None:
@@ -218,6 +301,7 @@ class Texture(Component):
             "bbox",
             "map_height",
             "map_width",
+            "rotation",
             "minimum_x",
             "minimum_y",
             "maximum_x",
@@ -236,10 +320,13 @@ class Texture(Component):
         """Generates weight files for textures. Each file is a numpy array of zeros and
             dtype uint8 (0-255).
 
-        Args:
+        Arguments:
             layer (Layer): Layer with textures and tags.
         """
-        size = (self.map_height, self.map_width)
+        if layer.tags is None:
+            size = (self.map_size, self.map_size)
+        else:
+            size = (self.map_rotated_size, self.map_rotated_size)
         postfix = "_weight.png" if not layer.exclude_weight else ".png"
         if layer.count == 0:
             filepaths = [os.path.join(self._weights_dir, layer.name + postfix)]
@@ -266,7 +353,7 @@ class Texture(Component):
     def layers(self, layers: list[Layer]) -> None:
         """Sets list of layers with textures and tags.
 
-        Args:
+        Arguments:
             layers (list[Layer]): List of layers.
         """
         self._layers = layers
@@ -286,7 +373,7 @@ class Texture(Component):
             ),
         )
 
-    # pylint: disable=no-member
+    # pylint: disable=no-member, R0912
     def draw(self) -> None:
         """Iterates over layers and fills them with polygons from OSM data."""
         layers = self.layers_by_priority()
@@ -297,7 +384,14 @@ class Texture(Component):
 
         cumulative_image = None
 
+        # Dictionary to store info layer data.
+        # Key is a layer.info_layer, value is a list of polygon points as tuples (x, y).
+        info_layer_data = defaultdict(list)
+
         for layer in layers:
+            if self.map.texture_settings.skip_drains and layer.usage == "drain":
+                self.logger.debug("Skipping layer %s because of the usage.", layer.name)
+                continue
             if not layer.tags:
                 self.logger.debug("Layer %s has no tags, there's nothing to draw.", layer.name)
                 continue
@@ -316,9 +410,23 @@ class Texture(Component):
 
             mask = cv2.bitwise_not(cumulative_image)
 
-            for polygon in self.polygons(layer.tags, layer.width):  # type: ignore
+            for polygon in self.objects_generator(  # type: ignore
+                layer.tags, layer.width, layer.info_layer
+            ):
+                if layer.info_layer:
+                    info_layer_data[layer.info_layer].append(
+                        self.np_to_polygon_points(polygon)  # type: ignore
+                    )
                 cv2.fillPoly(layer_image, [polygon], color=255)  # type: ignore
 
+            if layer.info_layer == "roads":
+                for linestring in self.objects_generator(
+                    layer.tags, layer.width, layer.info_layer, yield_linestrings=True
+                ):
+                    info_layer_data[f"{layer.info_layer}_polylines"].append(
+                        linestring  # type: ignore
+                    )
+
             output_image = cv2.bitwise_and(layer_image, mask)
 
             cumulative_image = cv2.bitwise_or(cumulative_image, output_image)
@@ -326,13 +434,27 @@ class Texture(Component):
             cv2.imwrite(layer_path, output_image)
             self.logger.debug("Texture %s saved.", layer_path)
 
+        # Save info layer data.
+        if os.path.isfile(self.info_layer_path):
+            self.logger.debug(
+                "File %s already exists, will update to avoid overwriting.", self.info_layer_path
+            )
+            with open(self.info_layer_path, "r", encoding="utf-8") as f:
+                info_layer_data.update(json.load(f))
+
+        with open(self.info_layer_path, "w", encoding="utf-8") as f:
+            json.dump(info_layer_data, f, ensure_ascii=False, indent=4)
+            self.logger.debug("Info layer data saved to %s.", self.info_layer_path)
+
         if cumulative_image is not None:
             self.draw_base_layer(cumulative_image)
 
-        if not self.light_version:
+        if self.map.texture_settings.dissolve and self.game.code != "FS22":
+            # FS22 has textures splitted into 4 sublayers, which leads to a very
+            # long processing time when dissolving them.
             self.dissolve()
         else:
-            self.logger.info("Skipping dissolve in light version of the map.")
+            self.logger.debug("Skipping dissolve in light version of the map.")
 
     def dissolve(self) -> None:
         """Dissolves textures of the layers with tags into sublayers for them to look more
@@ -376,12 +498,6 @@ class Texture(Component):
             for coord in non_zero_coords:
                 sublayers[np.random.randint(0, layer.count)][coord[0], coord[1]] = 255
 
-            # # ! Debug test if the sublayers are correct.
-            # if True:
-            #     combined = cv2.bitwise_or(*sublayers)
-            #     # Match the original image with the combined sublayers.
-            #     assert np.array_equal(layer_image, combined), "Sublayers are not correct."
-
             # Save the sublayers.
             for sublayer, sublayer_path in zip(sublayers, layer_paths):
                 cv2.imwrite(sublayer_path, sublayer)
@@ -389,13 +505,11 @@ class Texture(Component):
 
             self.logger.info("Dissolved layer %s.", layer.name)
 
-        self.logger.info("Dissolving finished.")
-
     def draw_base_layer(self, cumulative_image: np.ndarray) -> None:
         """Draws base layer and saves it into the png file.
         Base layer is the last layer to be drawn, it fills the remaining area of the map.
 
-        Args:
+        Arguments:
             cumulative_image (np.ndarray): Cumulative image with all layers.
         """
         base_layer = self.get_base_layer()
@@ -409,32 +523,45 @@ class Texture(Component):
     def get_relative_x(self, x: float) -> int:
         """Converts UTM X coordinate to relative X coordinate in map image.
 
-        Args:
+        Arguments:
             x (float): UTM X coordinate.
 
         Returns:
             int: Relative X coordinate in map image.
         """
-        return int(self.map_width * (x - self.minimum_x) / (self.maximum_x - self.minimum_x))
+        return int(self.map_rotated_size * (x - self.minimum_x) / (self.maximum_x - self.minimum_x))
 
     def get_relative_y(self, y: float) -> int:
         """Converts UTM Y coordinate to relative Y coordinate in map image.
 
-        Args:
+        Arguments:
             y (float): UTM Y coordinate.
 
         Returns:
             int: Relative Y coordinate in map image.
         """
-        return int(self.map_height * (1 - (y - self.minimum_y) / (self.maximum_y - self.minimum_y)))
+        return int(
+            self.map_rotated_size * (1 - (y - self.minimum_y) / (self.maximum_y - self.minimum_y))
+        )
+
+    def np_to_polygon_points(self, np_array: np.ndarray) -> list[tuple[int, int]]:
+        """Converts numpy array of polygon points to list of tuples.
+
+        Arguments:
+            np_array (np.ndarray): Numpy array of polygon points.
+
+        Returns:
+            list[tuple[int, int]]: List of tuples.
+        """
+        return [(int(x), int(y)) for x, y in np_array.reshape(-1, 2)]
 
     # pylint: disable=W0613
     def _to_np(self, geometry: shapely.geometry.polygon.Polygon, *args) -> np.ndarray:
         """Converts Polygon geometry to numpy array of polygon points.
 
-        Args:
+        Arguments:
             geometry (shapely.geometry.polygon.Polygon): Polygon geometry.
-            *args: Additional arguments:
+            *Arguments: Additional arguments:
                 - width (int | None): Width of the polygon in meters.
 
         Returns:
@@ -446,15 +573,17 @@ class Texture(Component):
         pairs = list(zip(xs, ys))
         return np.array(pairs, dtype=np.int32).reshape((-1, 1, 2))
 
-    def _to_polygon(self, obj: pd.core.series.Series, width: int | None) -> np.ndarray | None:
+    def _to_polygon(
+        self, obj: pd.core.series.Series, width: int | None
+    ) -> shapely.geometry.polygon.Polygon:
         """Converts OSM object to numpy array of polygon points.
 
-        Args:
+        Arguments:
             obj (pd.core.series.Series): OSM object.
             width (int | None): Width of the polygon in meters.
 
         Returns:
-            np.ndarray | None: Numpy array of polygon points.
+            shapely.geometry.polygon.Polygon: Polygon geometry.
         """
         geometry = obj["geometry"]
         geometry_type = geometry.geom_type
@@ -468,62 +597,129 @@ class Texture(Component):
         self,
         geometry: shapely.geometry.linestring.LineString | shapely.geometry.point.Point,
         width: int | None,
-    ) -> np.ndarray:
+    ) -> shapely.geometry.polygon.Polygon:
         """Converts LineString or Point geometry to numpy array of polygon points.
 
-        Args:
+        Arguments:
             geometry (shapely.geometry.linestring.LineString | shapely.geometry.point.Point):
                 LineString or Point geometry.
             width (int | None): Width of the polygon in meters.
 
         Returns:
-            np.ndarray: Numpy array of polygon points.
+            shapely.geometry.polygon.Polygon: Polygon geometry.
         """
         polygon = geometry.buffer(width)
-        return self._to_np(polygon)
+        return polygon
+
+    def _skip(
+        self, geometry: shapely.geometry.polygon.Polygon, *args, **kwargs
+    ) -> shapely.geometry.polygon.Polygon:
+        """Returns the same geometry.
+
+        Arguments:
+            geometry (shapely.geometry.polygon.Polygon): Polygon geometry.
+
+        Returns:
+            shapely.geometry.polygon.Polygon: Polygon geometry.
+        """
+        return geometry
 
     def _converters(
         self, geom_type: str
     ) -> Optional[Callable[[BaseGeometry, Optional[int]], np.ndarray]]:
         """Returns a converter function for a given geometry type.
 
-        Args:
+        Arguments:
             geom_type (str): Geometry type.
 
         Returns:
             Callable[[shapely.geometry, int | None], np.ndarray]: Converter function.
         """
-        converters = {"Polygon": self._to_np, "LineString": self._sequence, "Point": self._sequence}
+        converters = {"Polygon": self._skip, "LineString": self._sequence, "Point": self._sequence}
         return converters.get(geom_type)  # type: ignore
 
-    def polygons(
-        self, tags: dict[str, str | list[str] | bool], width: int | None
+    def objects_generator(
+        self,
+        tags: dict[str, str | list[str] | bool],
+        width: int | None,
+        info_layer: str | None = None,
+        yield_linestrings: bool = False,
+    ) -> Generator[np.ndarray, None, None] | Generator[list[tuple[int, int]], None, None]:
+        """Generator which yields numpy arrays of polygons from OSM data.
+
+        Arguments:
+            tags (dict[str, str | list[str]]): Dictionary of tags to search for.
+            width (int | None): Width of the polygon in meters (only for LineString).
+            info_layer (str | None): Name of the corresponding info layer.
+            yield_linestrings (bool): Flag to determine if the LineStrings should be yielded.
+
+        Yields:
+            Generator[np.ndarray, None, None] | Generator[list[tuple[int, int]], None, None]:
+                Numpy array of polygon points or list of point coordinates.
+        """
+        is_fieds = info_layer == "fields"
+        try:
+            if self.map.custom_osm is not None:
+                objects = ox.features_from_xml(self.map.custom_osm, tags=tags)
+            else:
+                objects = ox.features_from_bbox(bbox=self.new_bbox, tags=tags)
+        except Exception as e:  # pylint: disable=W0718
+            self.logger.debug("Error fetching objects for tags: %s. Error: %s.", tags, e)
+            return
+        objects_utm = ox.projection.project_gdf(objects, to_latlong=False)
+        self.logger.debug("Fetched %s elements for tags: %s.", len(objects_utm), tags)
+
+        method = self.linestrings_generator if yield_linestrings else self.polygons_generator
+
+        yield from method(objects_utm, width, is_fieds)
+
+    def linestrings_generator(
+        self, objects_utm: pd.core.frame.DataFrame, *args, **kwargs
+    ) -> Generator[list[tuple[int, int]], None, None]:
+        """Generator which yields lists of point coordinates which represent LineStrings from OSM.
+
+        Arguments:
+            objects_utm (pd.core.frame.DataFrame): Dataframe with OSM objects in UTM format.
+
+        Yields:
+            Generator[list[tuple[int, int]], None, None]: List of point coordinates.
+        """
+        for _, obj in objects_utm.iterrows():
+            geometry = obj["geometry"]
+            if isinstance(geometry, shapely.geometry.linestring.LineString):
+                points = [
+                    (self.get_relative_x(x), self.get_relative_y(y)) for x, y in geometry.coords
+                ]
+                yield points
+
+    def polygons_generator(
+        self, objects_utm: pd.core.frame.DataFrame, width: int | None, is_fieds: bool
     ) -> Generator[np.ndarray, None, None]:
         """Generator which yields numpy arrays of polygons from OSM data.
 
-        Args:
-            tags (dict[str, str | list[str]]): Dictionary of tags to search for.
+        Arguments:
+            objects_utm (pd.core.frame.DataFrame): Dataframe with OSM objects in UTM format.
             width (int | None): Width of the polygon in meters (only for LineString).
+            is_fieds (bool): Flag to determine if the fields should be padded.
 
         Yields:
             Generator[np.ndarray, None, None]: Numpy array of polygon points.
         """
-        try:
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore", DeprecationWarning)
-                objects = ox.features_from_bbox(bbox=self.bbox, tags=tags)
-        except Exception as e:  # pylint: disable=W0718
-            self.logger.warning("Error fetching objects for tags: %s.", tags)
-            self.logger.warning(e)
-            return
-        objects_utm = ox.project_gdf(objects, to_latlong=False)
-        self.logger.debug("Fetched %s elements for tags: %s.", len(objects_utm), tags)
-
         for _, obj in objects_utm.iterrows():
             polygon = self._to_polygon(obj, width)
             if polygon is None:
                 continue
-            yield polygon
+
+            if is_fieds and self.map.texture_settings.fields_padding > 0:
+                padded_polygon = polygon.buffer(-self.map.texture_settings.fields_padding)
+
+                if not isinstance(padded_polygon, shapely.geometry.polygon.Polygon):
+                    self.logger.warning("The padding value is too high, field will not padded.")
+                else:
+                    polygon = padded_polygon
+
+            polygon_np = self._to_np(polygon)
+            yield polygon_np
 
     def previews(self) -> list[str]:
         """Invokes methods to generate previews. Returns list of paths to previews.
@@ -542,13 +738,11 @@ class Texture(Component):
         Returns:
             str: Path to the preview.
         """
-        scaling_factor = min(
-            PREVIEW_MAXIMUM_SIZE / self.map_width, PREVIEW_MAXIMUM_SIZE / self.map_height
-        )
+        scaling_factor = PREVIEW_MAXIMUM_SIZE / self.map_size
 
         preview_size = (
-            int(self.map_width * scaling_factor),
-            int(self.map_height * scaling_factor),
+            int(self.map_size * scaling_factor),
+            int(self.map_size * scaling_factor),
         )
         self.logger.debug(
             "Scaling factor: %s. Preview size: %s.",
@@ -579,6 +773,7 @@ class Texture(Component):
             merged.dtype,
         )
         preview_path = os.path.join(self.previews_directory, "textures_osm.png")
-        cv2.imwrite(preview_path, merged)  # pylint: disable=no-member
-        self.logger.info("Preview saved to %s.", preview_path)
+
+        cv2.imwrite(preview_path, merged)  # type: ignore
+        self.logger.debug("Preview saved to %s.", preview_path)
         return preview_path
